@@ -6,16 +6,18 @@
 use std::cmp;
 use std::cmp::Ordering;
 use std::ops::Range;
-use std::rc::Rc;
 use std::str;
 use std::sync::Arc;
 use std::usize;
 
 use anyhow::Result;
 use bio::alignment::AlignmentOperation;
-use bio::stats::{self, pairhmm::GapParameters, pairhmm::PairHMM, LogProb, Prob};
+use bio::stats::{
+    pairhmm::EmissionParameters, pairhmm::GapParameters, pairhmm::PairHMM, LogProb, Prob,
+};
 use bio_types::genome;
 use bio_types::genome::AbstractInterval;
+use itertools::Itertools;
 use rust_htslib::bam;
 
 use crate::errors::Error;
@@ -31,24 +33,22 @@ pub(crate) mod pairhmm;
 
 use crate::variants::evidence::realignment::edit_distance::EditDistanceHit;
 
+use self::pairhmm::RefBaseEmission;
+use self::pairhmm::{ReadVsAlleleEmission, RefBaseVariantEmission};
+
 pub(crate) struct CandidateRegion {
     overlap: bool,
     read_interval: Range<usize>,
     ref_interval: Range<usize>,
 }
 
-pub(crate) trait Realignable<'a> {
-    type EmissionParams: stats::pairhmm::EmissionParameters
-        + pairhmm::RefBaseEmission
-        + pairhmm::VariantEmission;
-
+pub(crate) trait Realignable {
     fn alt_emission_params(
         &self,
-        read_emission_params: Rc<ReadEmission<'a>>,
         ref_buffer: Arc<reference::Buffer>,
         ref_interval: &genome::Interval,
         ref_window: usize,
-    ) -> Result<Vec<Self::EmissionParams>>;
+    ) -> Result<Vec<Box<dyn RefBaseVariantEmission>>>;
 
     /// Returns true if reads emitted from alt allele
     /// may be interpreted as revcomp reads by the mapper.
@@ -168,9 +168,10 @@ pub(crate) trait Realigner {
         record: &'a bam::Record,
         loci: L,
         variant: &V,
+        alt_variants: &[Box<dyn Realignable>],
     ) -> Result<AlleleSupport>
     where
-        V: Realignable<'a>,
+        V: Realignable,
         L: IntoIterator,
         L::Item: AsRef<SingleLocus>,
     {
@@ -238,36 +239,67 @@ pub(crate) trait Realigner {
 
         for region in merged_regions {
             // read emission
-            let read_emission = Rc::new(ReadEmission::new(
+            let read_emission = ReadEmission::new(
                 read_seq,
                 read_qual,
                 region.read_interval.start,
                 region.read_interval.end,
-            ));
+            );
             let mut edit_dist =
                 EditDistanceCalculation::new(region.read_interval.clone().map(|i| read_seq[i]));
 
-            // ref allele
-            let (mut prob_ref, _) = self.prob_allele(
-                &mut [ReferenceEmissionParams {
+            // Prepare reference alleles (the actual reference and any alt variants).
+            // METHOD: here, we consider all alternative variants, together with the reference allele.
+            // The method prob_allele below then chooses the most likely allele of origin via edit distance.
+            // On ties, it computes the full likelihoods via pair HMMs and chooses the maximum.
+            // This is an approximation of calculating full VAFs for all possible variants at a locus,
+            // under the assumption that in reality usually there will be just one of them with a high
+            // probability, and if not, the highest is still a good representative for comparison
+            // with the variant allele.
+            let mut ref_emissions = vec![ReadVsAlleleEmission::new(
+                &read_emission,
+                Box::new(ReferenceEmissionParams {
                     ref_seq: Arc::clone(&ref_seq),
                     ref_offset: region.ref_interval.start,
                     ref_end: region.ref_interval.end,
-                    read_emission: Rc::clone(&read_emission),
-                }],
-                &mut edit_dist,
-            );
+                }),
+            )];
+            for variant in alt_variants {
+                ref_emissions.extend(
+                    variant
+                        .alt_emission_params(
+                            Arc::clone(self.ref_buffer()),
+                            &genome::Interval::new(
+                                record.contig().to_owned(),
+                                region.ref_interval.start as u64..region.ref_interval.end as u64,
+                            ),
+                            self.ref_window(),
+                        )?
+                        .into_iter()
+                        .map(|allele_emission| {
+                            ReadVsAlleleEmission::new(&read_emission, allele_emission)
+                        }),
+                );
+            }
+
+            // ref allele (or one of the alt variants)
+            let (mut prob_ref, _) = self.prob_allele(&mut ref_emissions, &mut edit_dist);
 
             let (mut prob_alt, alt_hit) = self.prob_allele(
-                &mut variant.alt_emission_params(
-                    Rc::clone(&read_emission),
-                    Arc::clone(self.ref_buffer()),
-                    &genome::Interval::new(
-                        record.contig().to_owned(),
-                        region.ref_interval.start as u64..region.ref_interval.end as u64,
-                    ),
-                    self.ref_window(),
-                )?,
+                &mut variant
+                    .alt_emission_params(
+                        Arc::clone(self.ref_buffer()),
+                        &genome::Interval::new(
+                            record.contig().to_owned(),
+                            region.ref_interval.start as u64..region.ref_interval.end as u64,
+                        ),
+                        self.ref_window(),
+                    )?
+                    .into_iter()
+                    .map(|allele_emission| {
+                        ReadVsAlleleEmission::new(&read_emission, allele_emission)
+                    })
+                    .collect_vec(),
                 &mut edit_dist,
             );
 
@@ -341,14 +373,11 @@ pub(crate) trait Realigner {
     }
 
     /// Calculate probability of a certain allele.
-    fn prob_allele<E>(
+    fn prob_allele(
         &mut self,
-        candidate_allele_params: &mut [E],
+        candidate_allele_params: &mut [ReadVsAlleleEmission],
         edit_dist: &mut edit_distance::EditDistanceCalculation,
-    ) -> (LogProb, EditDistanceHit)
-    where
-        E: stats::pairhmm::EmissionParameters + pairhmm::RefBaseEmission + pairhmm::VariantEmission,
-    {
+    ) -> (LogProb, EditDistanceHit) {
         let mut hits = Vec::new();
         let mut best_dist = None;
         for params in candidate_allele_params.iter_mut() {
@@ -390,9 +419,11 @@ pub(crate) trait Realigner {
         (prob.unwrap(), best_hit.unwrap())
     }
 
-    fn calculate_prob_allele<E>(&mut self, hit: &EditDistanceHit, allele_params: &mut E) -> LogProb
-    where
-        E: stats::pairhmm::EmissionParameters + pairhmm::RefBaseEmission;
+    fn calculate_prob_allele(
+        &mut self,
+        hit: &EditDistanceHit,
+        allele_params: &mut ReadVsAlleleEmission,
+    ) -> LogProb;
 
     fn ref_buffer(&self) -> &Arc<reference::Buffer>;
 
@@ -433,10 +464,11 @@ impl Realigner for PairHMMRealigner {
         self.max_window
     }
 
-    fn calculate_prob_allele<E>(&mut self, hit: &EditDistanceHit, allele_params: &mut E) -> LogProb
-    where
-        E: stats::pairhmm::EmissionParameters + pairhmm::RefBaseEmission,
-    {
+    fn calculate_prob_allele(
+        &mut self,
+        hit: &EditDistanceHit,
+        allele_params: &mut ReadVsAlleleEmission,
+    ) -> LogProb {
         // METHOD: We shrink the area to run the HMM against to an environment around the best
         // edit distance hits.
         allele_params.shrink_to_hit(hit);
@@ -502,10 +534,11 @@ impl Realigner for PathHMMRealigner {
         self.max_window
     }
 
-    fn calculate_prob_allele<E>(&mut self, hit: &EditDistanceHit, allele_params: &mut E) -> LogProb
-    where
-        E: stats::pairhmm::EmissionParameters + pairhmm::RefBaseEmission,
-    {
+    fn calculate_prob_allele(
+        &mut self,
+        hit: &EditDistanceHit,
+        allele_params: &mut ReadVsAlleleEmission,
+    ) -> LogProb {
         let mut best_prob = None;
         for alignment in hit.alignments() {
             let mut prob = LogProb::ln_one();
@@ -581,13 +614,3 @@ impl Realigner for PathHMMRealigner {
         best_prob.unwrap()
     }
 }
-
-pub(crate) trait AltAlleleEmissionBuilder {
-    type EmissionParams: stats::pairhmm::EmissionParameters + pairhmm::RefBaseEmission;
-
-    fn build<'a>(
-        &self,
-        read_emission_params: &'a ReadEmission,
-        ref_seq: &'a [u8],
-    ) -> Self::EmissionParams;
-}
